searchData={"items":[{"type":"module","title":"Authentication","doc":"","ref":"Authentication.html"},{"type":"function","title":"Authentication.definition/0","doc":"","ref":"Authentication.html#definition/0"},{"type":"module","title":"ExMachine","doc":"ExMachine - A functional implementation of hierarchical state machines based on Statechart formalism.\n\nExMachine provides a purely functional approach to defining and executing finite state machines\nthat follow the Statechart specification proposed by David Harel in 1987. It supports\nhierarchical states, entry/exit actions, guard functions, and extended state management.","ref":"ExMachine.html"},{"type":"module","title":"Features - ExMachine","doc":"- **Hierarchical States**: States can contain substates, creating complex state hierarchies\n- **Entry/Exit Actions**: Execute functions when entering or leaving states\n- **Transition Actions**: Execute functions during state transitions\n- **Guard Functions**: Conditional logic to control when transitions can occur\n- **Extended State**: Maintain context data that travels with the state machine\n- **Internal Events**: Support for run-to-completion semantics\n- **Compile-time Validation**: State machine definitions are validated at compile time","ref":"ExMachine.html#module-features"},{"type":"module","title":"Basic Usage - ExMachine","doc":"# Define a simple state machine\n    defmodule TrafficLight do\n      use ExMachine.Statechart\n\n      alias ExMachine.State\n\n      def definition do\n        %State{\n          initial: \"red\",\n          substates: %{\n            \"red\" => %State{transitions: %{\"timer\" => \"green\"}},\n            \"green\" => %State{transitions: %{\"timer\" => \"yellow\"}},\n            \"yellow\" => %State{transitions: %{\"timer\" => \"red\"}}\n          }\n        }\n      end\n    end\n\n    # Use the state machine\n    alias ExMachine.{Machine, Statechart}\n\n    statechart = Statechart.build(TrafficLight.definition())\n    machine = ExMachine.init(statechart, %{})\n    machine = ExMachine.dispatch(machine, \"timer\")","ref":"ExMachine.html#module-basic-usage"},{"type":"module","title":"State Machine Execution - ExMachine","doc":"State machines in ExMachine follow these principles:\n\n1. **Immutable**: Each transition returns a new machine instance\n2. **Functional**: No side effects during transitions (except through actions)\n3. **Deterministic**: Given the same state and event, the result is always the same\n4. **Run-to-completion**: Events are processed completely before the next event\n\nSee the main modules for detailed documentation:\n\n- `ExMachine.Machine` - Core machine execution logic\n- `ExMachine.Statechart` - State machine definition and validation\n- `ExMachine.State` - Individual state definitions\n- `ExMachine.Transition` - Transition definitions with actions and guards","ref":"ExMachine.html#module-state-machine-execution"},{"type":"function","title":"ExMachine.dispatch/2","doc":"Dispatch an event to a running state machine.\n\nThis is a convenience function that delegates to `ExMachine.Machine.dispatch/2`.","ref":"ExMachine.html#dispatch/2"},{"type":"function","title":"Parameters - ExMachine.dispatch/2","doc":"- `machine` - A running `ExMachine.Machine` instance\n- `event` - The event to dispatch (string or atom)","ref":"ExMachine.html#dispatch/2-parameters"},{"type":"function","title":"Returns - ExMachine.dispatch/2","doc":"A new `ExMachine.Machine` instance with the updated state and context.","ref":"ExMachine.html#dispatch/2-returns"},{"type":"function","title":"Examples - ExMachine.dispatch/2","doc":"machine = ExMachine.dispatch(machine, \"start\")\n    machine = ExMachine.dispatch(machine, :stop)","ref":"ExMachine.html#dispatch/2-examples"},{"type":"function","title":"ExMachine.init/2","doc":"Initialize a state machine with the given statechart definition and initial context.\n\nThis is a convenience function that delegates to `ExMachine.Machine.init/2`.","ref":"ExMachine.html#init/2"},{"type":"function","title":"Parameters - ExMachine.init/2","doc":"- `statechart` - A compiled statechart definition from `ExMachine.Statechart.build/1`\n- `context` - Initial context data (any term)","ref":"ExMachine.html#init/2-parameters"},{"type":"function","title":"Returns - ExMachine.init/2","doc":"A running `ExMachine.Machine` in its initial configuration.","ref":"ExMachine.html#init/2-returns"},{"type":"function","title":"Examples - ExMachine.init/2","doc":"statechart = Statechart.build(MyStateMachine.definition())\n    machine = ExMachine.init(statechart, %{counter: 0})","ref":"ExMachine.html#init/2-examples"},{"type":"module","title":"ExMachine.Context","doc":"Utility functions for managing statechart execution context.\n\nThe context is a map that holds the current state of data during statechart\nexecution. It serves as the primary means of data storage and communication\nbetween different parts of the state machine, including actions, guards,\nand transitions.\n\nThis module provides convenient wrapper functions around standard Map operations\nthat are commonly used when working with statechart contexts. All functions\nare designed to be imported into state modules for ease of use.","ref":"ExMachine.Context.html"},{"type":"module","title":"Context Structure - ExMachine.Context","doc":"The context is a regular Elixir map with some reserved keys used internally\nby ExMachine:\n\n* `:exm_params` - Parameters passed with the current event\n* `:exm_queue` - Internal event queue for raised events\n\nAll other keys are available for application-specific data.","ref":"ExMachine.Context.html#module-context-structure"},{"type":"module","title":"Examples - ExMachine.Context","doc":"# Basic context operations\n    context = %{}\n    context = put(context, :user_id, 123)\n    context = put(context, :status, :active)\n\n    user_id = get(context, :user_id)  # => 123\n    status = get(context, :status, :inactive)  # => :active\n\n    context = delete(context, :status)\n\n    # Working with event parameters\n    context = put_params(context, %{action: :login, user: \"john\"})\n    params = get_params(context)  # => %{action: :login, user: \"john\"}\n\n    # Raising internal events\n    context = raise_event(context, :auto_save)\n    context = raise_event(context, {:notify, \"Task completed\"})","ref":"ExMachine.Context.html#module-examples"},{"type":"module","title":"Usage in State Definitions - ExMachine.Context","doc":"defmodule UserAuthStatechart do\n      use ExMachine.Statechart,\n        definition: %{\n          initial: \"logged_out\",\n          states: %{\n            \"logged_out\" => %{\n              entry: fn context ->\n                context\n                |> delete(:user_id)\n                |> delete(:session_token)\n              end,\n              transitions: %{\n                \"login\" => %{\n                  target: \"logged_in\",\n                  action: fn context ->\n                    params = get_params(context)\n                    context\n                    |> put(:user_id, params.user_id)\n                    |> put(:session_token, generate_token())\n                    |> raise_event(:session_started)\n                  end\n                }\n              }\n            },\n            \"logged_in\" => %{\n              transitions: %{\n                \"logout\" => \"logged_out\"\n              }\n            }\n          }\n        }\n    end","ref":"ExMachine.Context.html#module-usage-in-state-definitions"},{"type":"function","title":"ExMachine.Context.delete/2","doc":"Delete a key from the context.\n\nSame as `Map.delete/2`, useful because it's imported in state modules.","ref":"ExMachine.Context.html#delete/2"},{"type":"function","title":"Examples - ExMachine.Context.delete/2","doc":"iex> context = put(%{}, :hello, :world)\n    iex> context[:hello]\n    :world\n    iex> context = delete(context, :hello)\n    iex> context[:hello]\n    nil","ref":"ExMachine.Context.html#delete/2-examples"},{"type":"function","title":"ExMachine.Context.delete_params/1","doc":"Remove event parameters from context.\n\nThis is typically done automatically by the state machine engine\nafter processing an event.","ref":"ExMachine.Context.html#delete_params/1"},{"type":"function","title":"Examples - ExMachine.Context.delete_params/1","doc":"iex> context = put_params(%{}, %{id: 1})\n    iex> context = delete_params(context)\n    iex> get_params(context)\n    nil","ref":"ExMachine.Context.html#delete_params/1-examples"},{"type":"function","title":"ExMachine.Context.get/3","doc":"Get the value of key in context.\n\nSame as `Map.get/3`, useful because it's imported in state modules.","ref":"ExMachine.Context.html#get/3"},{"type":"function","title":"Examples - ExMachine.Context.get/3","doc":"iex> context = put(%{}, :hello, :world)\n    iex> get(context, :hello)\n    :world\n\n    iex> get(%{}, :missing_key, :default_value)\n    :default_value","ref":"ExMachine.Context.html#get/3-examples"},{"type":"function","title":"ExMachine.Context.get_params/1","doc":"Get the current event parameters from context.\n\nReturns `nil` if no parameters were set for the current event.","ref":"ExMachine.Context.html#get_params/1"},{"type":"function","title":"Examples - ExMachine.Context.get_params/1","doc":"iex> context = put_params(%{}, %{id: 1})\n    iex> get_params(context)\n    %{id: 1}\n\n    iex> get_params(%{})\n    nil","ref":"ExMachine.Context.html#get_params/1-examples"},{"type":"function","title":"ExMachine.Context.put/3","doc":"Puts the given value under key in context.\n\nSame as `Map.put/3`, useful because it's imported in state modules.","ref":"ExMachine.Context.html#put/3"},{"type":"function","title":"Examples - ExMachine.Context.put/3","doc":"iex> context = put(%{}, :hello, :world)\n    iex> context[:hello]\n    :world","ref":"ExMachine.Context.html#put/3-examples"},{"type":"function","title":"ExMachine.Context.put_params/2","doc":"Puts the given term under the event params key in context.\n\nEvent parameters are typically provided when sending events to the state machine\nand can be accessed by actions and guards during transition processing.","ref":"ExMachine.Context.html#put_params/2"},{"type":"function","title":"Examples - ExMachine.Context.put_params/2","doc":"iex> context = put_params(%{}, %{user_id: 123, action: :update})\n    iex> get_params(context)\n    %{user_id: 123, action: :update}","ref":"ExMachine.Context.html#put_params/2-examples"},{"type":"function","title":"ExMachine.Context.raise_event/2","doc":"Raise an internal event during the execution of an action.\n\nInternal events are processed immediately after the current macrostep\ncompletes, allowing actions to trigger additional transitions without\nrequiring external events.","ref":"ExMachine.Context.html#raise_event/2"},{"type":"function","title":"Parameters - ExMachine.Context.raise_event/2","doc":"* `context` - The current execution context\n* `event` - The event to raise (atom or tuple with parameters)","ref":"ExMachine.Context.html#raise_event/2-parameters"},{"type":"function","title":"Examples - ExMachine.Context.raise_event/2","doc":"# Simple internal event\n    context = raise_event(context, :auto_save)\n\n    # Internal event with parameters\n    context = raise_event(context, {:notify, %{message: \"Process completed\"}})\n\n    # Multiple internal events can be queued\n    context = context\n              |> raise_event(:validate_data)\n              |> raise_event(:send_notification)","ref":"ExMachine.Context.html#raise_event/2-examples"},{"type":"function","title":"Usage in Actions - ExMachine.Context.raise_event/2","doc":"action: fn context ->\n      if should_auto_logout?(context) do\n        raise_event(context, :auto_logout)\n      else\n        context\n      end\n    end","ref":"ExMachine.Context.html#raise_event/2-usage-in-actions"},{"type":"module","title":"ExMachine.Final","doc":"Represents a final state in a statechart.\n\nA final state represents a completed region of a statechart. When a state machine\nenters a final state, it indicates that the associated region has completed its\nintended activity. In hierarchical state machines, entering a final state may\ntrigger completion events that can be used by parent states.","ref":"ExMachine.Final.html"},{"type":"module","title":"Fields - ExMachine.Final","doc":"* `:entry` - Optional function to execute when entering the final state.\n  This is typically used for cleanup actions or to signal the completion\n  of the state machine's purpose.","ref":"ExMachine.Final.html#module-fields"},{"type":"module","title":"Examples - ExMachine.Final","doc":"# Define a final state with an entry action\n    %ExMachine.Final{\n      entry: fn context ->\n        # Log completion or cleanup resources\n        IO.puts(\"Process completed successfully\")\n        context\n      end\n    }\n\n    # Simple final state without entry action\n    %ExMachine.Final{}","ref":"ExMachine.Final.html#module-examples"},{"type":"module","title":"Usage in Statechart - ExMachine.Final","doc":"Final states are typically used to represent the successful completion of a\nworkflow or process:\n\n    defmodule WorkflowStatechart do\n      use ExMachine.Statechart,\n        definition: %{\n          initial: \"processing\",\n          states: %{\n            \"processing\" => %{\n              transitions: %{\"completed\" => \"done\"}\n            },\n            \"done\" => %ExMachine.Final{\n              entry: fn context ->\n                send_completion_notification(context)\n                context\n              end\n            }\n          }\n        }\n    end","ref":"ExMachine.Final.html#module-usage-in-statechart"},{"type":"type","title":"ExMachine.Final.t/0","doc":"","ref":"ExMachine.Final.html#t:t/0"},{"type":"module","title":"ExMachine.History","doc":"Represents a history state in a statechart.\n\nHistory states provide a way to remember the most recent active state\nconfiguration when exiting a composite state, allowing the state machine\nto resume from where it left off when re-entering that region.\n\nExMachine supports two types of history states:\n\n* `:shallow` - Remembers only the immediate child state that was active\n* `:deep` - Remembers the entire active state configuration, including\n  nested states at all levels","ref":"ExMachine.History.html"},{"type":"module","title":"Fields - ExMachine.History","doc":"* `:type` - The type of history state (`:shallow` or `:deep`)","ref":"ExMachine.History.html#module-fields"},{"type":"module","title":"Examples - ExMachine.History","doc":"# Shallow history - remembers only immediate child\n    %ExMachine.History{type: :shallow}\n\n    # Deep history - remembers full nested configuration\n    %ExMachine.History{type: :deep}","ref":"ExMachine.History.html#module-examples"},{"type":"module","title":"Usage in Statechart - ExMachine.History","doc":"History states are particularly useful in scenarios where you want to\npreserve user context when temporarily leaving a region:\n\n    defmodule MediaPlayerStatechart do\n      use ExMachine.Statechart,\n        definition: %{\n          initial: \"stopped\",\n          states: %{\n            \"stopped\" => %{\n              transitions: %{\"play\" => \"playing\"}\n            },\n            \"playing\" => %{\n              initial: \"normal_speed\",\n              transitions: %{\n                \"pause\" => \"paused\",\n                \"stop\" => \"stopped\"\n              },\n              states: %{\n                \"normal_speed\" => %{\n                  transitions: %{\"fast_forward\" => \"fast\"}\n                },\n                \"fast\" => %{\n                  transitions: %{\"normal\" => \"normal_speed\"}\n                }\n              }\n            },\n            \"paused\" => %{\n              transitions: %{\n                \"play\" => \"playing.history\",  # Resume from where paused\n                \"stop\" => \"stopped\"\n              }\n            },\n            \"playing.history\" => %ExMachine.History{type: :deep}\n          }\n        }\n    end\n\nIn this example, when the media player is paused and then resumed,\nit will return to the exact same playback mode (normal or fast forward)\nthat was active before pausing.","ref":"ExMachine.History.html#module-usage-in-statechart"},{"type":"type","title":"ExMachine.History.t/0","doc":"","ref":"ExMachine.History.html#t:t/0"},{"type":"module","title":"ExMachine.Machine","doc":"Core module for state machine execution and management.\n\nThe `Machine` module represents a running state machine instance. It maintains the current\nstate configuration, context data, execution history, and provides the core execution engine\nfor processing events and managing state transitions.","ref":"ExMachine.Machine.html"},{"type":"module","title":"Machine Structure - ExMachine.Machine","doc":"A machine contains:\n\n- `statechart` - The compiled state machine definition\n- `configuration` - Current active states (hierarchical paths)\n- `running?` - Whether the machine is currently running\n- `macrosteps` - History of transition sequences\n- `queue` - Queue of internal events to process\n- `context` - Extended state data that travels with the machine\n- `states_histories` - History states for complex statecharts","ref":"ExMachine.Machine.html#module-machine-structure"},{"type":"module","title":"State Configuration - ExMachine.Machine","doc":"Unlike simple finite state machines, a statechart machine can be in multiple states\nsimultaneously due to hierarchical composition. The configuration represents all\ncurrently active states as a list of state paths from leaf states to the root.\n\nFor example, if a machine is in state \"playing/normal_speed\", the configuration\nwould be `[[\"normal_speed\", \"playing\", \"root\"]]`.","ref":"ExMachine.Machine.html#module-state-configuration"},{"type":"module","title":"Event Processing - ExMachine.Machine","doc":"Events are processed using run-to-completion semantics:\n\n1. An external event is received\n2. All possible transitions are evaluated\n3. Actions are executed and may generate internal events\n4. Internal events are processed until no more transitions are possible\n5. The machine reaches a stable configuration","ref":"ExMachine.Machine.html#module-event-processing"},{"type":"module","title":"Examples - ExMachine.Machine","doc":"# Initialize a machine\n    statechart = Statechart.build(MyMachine.definition())\n    machine = Machine.init(statechart, %{counter: 0})\n\n    # Process events\n    machine = Machine.dispatch(machine, \"start\")\n    machine = Machine.dispatch(machine, \"increment\")\n\n    # Check current state\n    IO.inspect(machine.configuration)\n    IO.inspect(machine.context)","ref":"ExMachine.Machine.html#module-examples"},{"type":"function","title":"ExMachine.Machine.dispatch/2","doc":"Dispatch an event to the machine, performing all the necessary activities\nand returning the machine in the new active configuration.\n\nIf during execution some activities raises internal events, the machine\ncontinues execution of these events until the internal queue is empty\n(run to completion)","ref":"ExMachine.Machine.html#dispatch/2"},{"type":"function","title":"ExMachine.Machine.get_active_states/1","doc":"Returns list of currents active states.\n\nGenerally, there is only one active state if the machine is not in a parallel\n(orthogonal) state, otherwise its the list of active states\nof each active region","ref":"ExMachine.Machine.html#get_active_states/1"},{"type":"function","title":"ExMachine.Machine.init/2","doc":"Initialize the machine, performing all the necessary activities\nand returning the machine in its initial active configuration\n\nThe log of the transition to initial state are recorded in the first\nMacrostep","ref":"ExMachine.Machine.html#init/2"},{"type":"function","title":"ExMachine.Machine.last_macrostep/1","doc":"Returns last executed macrostep","ref":"ExMachine.Machine.html#last_macrostep/1"},{"type":"function","title":"ExMachine.Machine.last_microsteps/1","doc":"Returns last executed list of microstep","ref":"ExMachine.Machine.html#last_microsteps/1"},{"type":"function","title":"ExMachine.Machine.last_transitions/1","doc":"Returns last executed list of transitions","ref":"ExMachine.Machine.html#last_transitions/1"},{"type":"function","title":"ExMachine.Machine.new/2","doc":"","ref":"ExMachine.Machine.html#new/2"},{"type":"type","title":"ExMachine.Machine.t/0","doc":"","ref":"ExMachine.Machine.html#t:t/0"},{"type":"module","title":"ExMachine.Macrostep","doc":"Represents a macrostep in statechart execution.\n\nA macrostep consists of a sequence (a chain) of microsteps, at the end of which\nthe state machine is in a stable state and ready to process an external event.\nEach external event causes an ExMachine state machine to take exactly one macrostep.\nHowever, if the external event does not enable any transitions, no microstep\nwill be taken, and the corresponding macrostep will be empty.","ref":"ExMachine.Macrostep.html"},{"type":"module","title":"Definition (From SCXML) - ExMachine.Macrostep","doc":"A macrostep consists of a sequence (a chain) of microsteps, at the end of which\nthe state machine is in a stable state and ready to process an external event.\nEach external event causes an SCXML state machine to take exactly one macrostep.\nHowever, if the external event does not enable any transitions, no microstep\nwill be taken, and the corresponding macrostep will be empty.","ref":"ExMachine.Macrostep.html#module-definition-from-scxml"},{"type":"module","title":"Definition (From SISMIC) - ExMachine.Macrostep","doc":"A macro step corresponds to the process of consuming an event, regardless of\nthe number and the type (eventless or not) of triggered transitions.\nA macro step also includes every consecutive stabilization step\n(i.e., the steps that are needed to enter nested states, or to switch into\nthe configuration of a history state).","ref":"ExMachine.Macrostep.html#module-definition-from-sismic"},{"type":"module","title":"Fields - ExMachine.Macrostep","doc":"* `:timestamp` - When the macrostep started execution\n* `:event` - The external event that triggered this macrostep\n* `:transitions` - List of all transitions taken during this macrostep\n* `:entered` - List of all states entered during this macrostep\n* `:exited` - List of all states exited during this macrostep\n* `:actions` - List of all actions executed during this macrostep\n* `:microsteps` - List of microsteps that comprise this macrostep","ref":"ExMachine.Macrostep.html#module-fields"},{"type":"module","title":"Important Notes - ExMachine.Macrostep","doc":"* The microsteps list is in reverse order of execution (LIFO structure)\n  for performance reasons. The first element in the list is the last\n  executed microstep.\n* A macrostep may contain multiple microsteps if internal events are\n  raised or if eventless transitions are triggered.","ref":"ExMachine.Macrostep.html#module-important-notes"},{"type":"module","title":"Examples - ExMachine.Macrostep","doc":"# Create a new macrostep\n    macrostep = ExMachine.Macrostep.new()\n\n    # A typical macrostep after processing a \"login\" event might look like:\n    %ExMachine.Macrostep{\n      timestamp: ~N[2023-06-09 10:30:45],\n      event: {:login, %{username: \"john\", password: \"secret\"}},\n      transitions: [\"logged_out -> logged_in\"],\n      entered: [\"logged_in\"],\n      exited: [\"logged_out\"],\n      actions: [:validate_credentials, :create_session],\n      microsteps: [\n        %ExMachine.Microstep{...},  # Last executed (reverse order)\n        %ExMachine.Microstep{...}   # First executed\n      ]\n    }","ref":"ExMachine.Macrostep.html#module-examples"},{"type":"module","title":"Usage in Debugging and Monitoring - ExMachine.Macrostep","doc":"Macrosteps provide a complete audit trail of state machine execution,\nmaking them invaluable for debugging, monitoring, and understanding\nthe behavior of complex statecharts:\n\n    # In a GenServer-based state machine\n    def handle_event(event, state) do\n      {new_state, macrostep} = ExMachine.Machine.process_event(state, event)\n\n      # Log macrostep for debugging\n      Logger.info(\"Macrostep completed\", %{\n        event: macrostep.event,\n        transitions: length(macrostep.transitions),\n        states_entered: macrostep.entered,\n        states_exited: macrostep.exited,\n        duration: NaiveDateTime.diff(NaiveDateTime.utc_now(), macrostep.timestamp, :microsecond)\n      })\n\n      {:noreply, new_state}\n    end","ref":"ExMachine.Macrostep.html#module-usage-in-debugging-and-monitoring"},{"type":"function","title":"ExMachine.Macrostep.new/0","doc":"Create a new macrostep with current timestamp.","ref":"ExMachine.Macrostep.html#new/0"},{"type":"function","title":"Examples - ExMachine.Macrostep.new/0","doc":"iex> macrostep = ExMachine.Macrostep.new()\n    iex> is_struct(macrostep, ExMachine.Macrostep)\n    true\n    iex> is_struct(macrostep.timestamp, NaiveDateTime)\n    true","ref":"ExMachine.Macrostep.html#new/0-examples"},{"type":"type","title":"ExMachine.Macrostep.t/0","doc":"","ref":"ExMachine.Macrostep.html#t:t/0"},{"type":"module","title":"ExMachine.Microstep","doc":"Represents an atomic execution step in statechart processing.\n\nA microstep is the smallest, atomic step that a statechart can execute.\nIt involves the processing of a single transition (or, in the case of\nparallel states, a single set of transitions). A microstep may change\nthe active configuration, update the data model and/or generate new\n(internal and/or external) events.","ref":"ExMachine.Microstep.html"},{"type":"module","title":"Definition (From SCXML) - ExMachine.Microstep","doc":"A microstep involves the processing of a single transition (or, in the case\nof parallel states, a single set of transitions.) A microstep may change\nthe active configuration, update the data model and/or generate new\n(internal and/or external) events. This, by causality, may in turn enable\nadditional transitions which will be handled in the next microstep in the\nsequence, and so on.","ref":"ExMachine.Microstep.html#module-definition-from-scxml"},{"type":"module","title":"Definition (From SISMIC) - ExMachine.Microstep","doc":"The smallest, atomic step that a statechart can execute. A step considers\nan event, takes a transition and results in a list of entered states and\na list of exited states. Order in the two lists is REALLY important!","ref":"ExMachine.Microstep.html#module-definition-from-sismic"},{"type":"module","title":"Fields - ExMachine.Microstep","doc":"* `:params` - Event parameters or `nil` for eventless transitions.\n  If event has parameters, the term is in the form `{event, params}`\n* `:transition` - The `ExMachine.Transition` processed or `nil` if no transition\n* `:entered` - Ordered list of states entered during this microstep\n* `:exited` - Ordered list of states exited during this microstep\n* `:actions` - List of actions executed during this microstep","ref":"ExMachine.Microstep.html#module-fields"},{"type":"module","title":"State Order Importance - ExMachine.Microstep","doc":"The order of states in the `:entered` and `:exited` lists is critical\nfor correct statechart behavior:\n\n* **Exit order**: States are exited from most nested to least nested\n  (children before parents)\n* **Entry order**: States are entered from least nested to most nested\n  (parents before children)\n\nThis ensures that exit actions run before entry actions, and that\nparent states are properly initialized before their children.","ref":"ExMachine.Microstep.html#module-state-order-importance"},{"type":"module","title":"Examples - ExMachine.Microstep","doc":"# Microstep for a simple transition\n    %ExMachine.Microstep{\n      params: {:login, %{username: \"john\"}},\n      transition: %ExMachine.Transition{\n        source: \"logged_out\",\n        target: \"logged_in\",\n        event: \"login\"\n      },\n      entered: [\"logged_in\"],\n      exited: [\"logged_out\"],\n      actions: [:validate_credentials, :create_session]\n    }\n\n    # Microstep for entering nested states\n    %ExMachine.Microstep{\n      params: {:start_game, %{level: 1}},\n      transition: %ExMachine.Transition{\n        source: \"menu\",\n        target: \"playing.level1.normal\"\n      },\n      entered: [\"playing\", \"level1\", \"normal\"],  # Parent to child order\n      exited: [\"menu\"],\n      actions: [:initialize_game, :load_level]\n    }\n\n    # Eventless microstep (internal transition)\n    %ExMachine.Microstep{\n      params: nil,\n      transition: %ExMachine.Transition{\n        source: \"processing\",\n        target: \"completed\"\n      },\n      entered: [\"completed\"],\n      exited: [\"processing\"],\n      actions: [:finalize_process]\n    }","ref":"ExMachine.Microstep.html#module-examples"},{"type":"module","title":"Usage in Debugging - ExMachine.Microstep","doc":"Microsteps provide detailed information about individual state changes,\nmaking them invaluable for debugging statechart behavior:\n\n    # Example function to analyze microstep execution\n    defp analyze_microstep(microstep) do\n      IO.puts(\"Microstep Analysis:\")\n      IO.puts(\"  Event: #{inspect(microstep.params)}\")\n      if microstep.transition do\n        IO.puts(\"  Transition: #{microstep.transition.source} -> #{microstep.transition.target}\")\n      end\n      IO.puts(\"  States exited: #{inspect(microstep.exited)}\")\n      IO.puts(\"  States entered: #{inspect(microstep.entered)}\")\n      IO.puts(\"  Actions executed: #{inspect(microstep.actions)}\")\n    end","ref":"ExMachine.Microstep.html#module-usage-in-debugging"},{"type":"function","title":"ExMachine.Microstep.new/0","doc":"Create a new empty microstep.","ref":"ExMachine.Microstep.html#new/0"},{"type":"function","title":"Examples - ExMachine.Microstep.new/0","doc":"iex> microstep = ExMachine.Microstep.new()\n    iex> is_struct(microstep, ExMachine.Microstep)\n    true\n    iex> microstep.entered\n    []\n    iex> microstep.exited\n    []","ref":"ExMachine.Microstep.html#new/0-examples"},{"type":"type","title":"ExMachine.Microstep.t/0","doc":"","ref":"ExMachine.Microstep.html#t:t/0"},{"type":"module","title":"ExMachine.ServerMachine","doc":"GenServer-based statechart execution server.\n\nThis module provides a GenServer wrapper around ExMachine statecharts,\nallowing statecharts to run as supervised processes that can handle\nevents asynchronously and maintain their state across multiple interactions.","ref":"ExMachine.ServerMachine.html"},{"type":"module","title":"Features - ExMachine.ServerMachine","doc":"* **Supervised Process**: Runs as a GenServer that can be supervised\n* **Asynchronous Event Processing**: Handle events without blocking callers\n* **State Persistence**: Maintains statechart state across events\n* **Event Queuing**: Built-in event queue for handling concurrent events\n* **Monitoring**: Built-in logging and monitoring capabilities\n* **Fault Tolerance**: Automatic restart and recovery through supervisor trees","ref":"ExMachine.ServerMachine.html#module-features"},{"type":"module","title":"Usage - ExMachine.ServerMachine","doc":"To use ExMachine.ServerMachine, you typically create a module that defines\nyour statechart and then start it as a GenServer:\n\n    defmodule MyStatechartServer do\n      use ExMachine.ServerMachine\n\n      # Define your statechart\n      def init_statechart() do\n        %{\n          initial: \"idle\",\n          states: %{\n            \"idle\" => %{\n              transitions: %{\n                \"start\" => \"working\"\n              }\n            },\n            \"working\" => %{\n              transitions: %{\n                \"complete\" => \"idle\",\n                \"error\" => \"error\"\n              }\n            },\n            \"error\" => %{\n              transitions: %{\n                \"reset\" => \"idle\"\n              }\n            }\n          }\n        }\n      end\n\n      # Optional: customize initial context\n      def init_context() do\n        %{\n          start_time: System.system_time(:second),\n          error_count: 0\n        }\n      end\n    end\n\n    # Start the server\n    {:ok, pid} = MyStatechartServer.start_link(name: :my_statechart)\n\n    # Send events\n    MyStatechartServer.send_event(pid, :start)\n    MyStatechartServer.send_event(pid, {:complete, %{result: \"success\"}})\n\n    # Get current state\n    current_state = MyStatechartServer.get_state(pid)","ref":"ExMachine.ServerMachine.html#module-usage"},{"type":"module","title":"Supervision - ExMachine.ServerMachine","doc":"ExMachine.ServerMachine processes can be easily supervised:\n\n    defmodule MyApp.Supervisor do\n      use Supervisor\n\n      def start_link(init_arg) do\n        Supervisor.start_link(__MODULE__, init_arg, name: __MODULE__)\n      end\n\n      def init(_init_arg) do\n        children = [\n          {MyStatechartServer, name: :my_statechart}\n        ]\n\n        Supervisor.init(children, strategy: :one_for_one)\n      end\n    end","ref":"ExMachine.ServerMachine.html#module-supervision"},{"type":"module","title":"Advanced Features - ExMachine.ServerMachine","doc":"","ref":"ExMachine.ServerMachine.html#module-advanced-features"},{"type":"module","title":"Custom Event Handling - ExMachine.ServerMachine","doc":"defmodule AdvancedStatechartServer do\n      use ExMachine.ServerMachine\n\n      # Override handle_event for custom processing\n      def handle_event(event, state) do\n        # Custom preprocessing\n        event = preprocess_event(event)\n\n        # Call default handler\n        result = super(event, state)\n\n        # Custom postprocessing\n        postprocess_result(result)\n      end\n    end","ref":"ExMachine.ServerMachine.html#module-custom-event-handling"},{"type":"module","title":"State Change Callbacks - ExMachine.ServerMachine","doc":"defmodule CallbackStatechartServer do\n      use ExMachine.ServerMachine\n\n      # Called whenever state changes\n      def on_state_change(old_config, new_config, context) do\n        Logger.info(\"State changed from #{inspect(old_config)} to #{inspect(new_config)}\")\n      end\n\n      # Called on macrostep completion\n      def on_macrostep_complete(completed_macrostep) do\n        Metrics.increment(\"statechart.macrosteps\")\n        Metrics.histogram(\"statechart.transition_count\", length(completed_macrostep.transitions))\n      end\n    end","ref":"ExMachine.ServerMachine.html#module-state-change-callbacks"},{"type":"module","title":"Error Handling - ExMachine.ServerMachine","doc":"The ServerMachine includes built-in error handling for common scenarios:\n\n* **Invalid Events**: Events that don't trigger any transitions are ignored\n* **Action Failures**: Failed actions are logged but don't crash the server\n* **Guard Failures**: Failed guards prevent transitions but don't crash\n* **Malformed Events**: Invalid event formats are logged and ignored","ref":"ExMachine.ServerMachine.html#module-error-handling"},{"type":"module","title":"Performance Considerations - ExMachine.ServerMachine","doc":"* Events are processed sequentially to maintain statechart semantics\n* Large contexts should avoid deep copying by using references\n* Consider using `cast` for fire-and-forget events\n* Use `call` when you need confirmation of event processing","ref":"ExMachine.ServerMachine.html#module-performance-considerations"},{"type":"module","title":"NOTE - ExMachine.ServerMachine","doc":"This module is currently a placeholder and needs implementation.\nThe documentation above represents the intended API and functionality.","ref":"ExMachine.ServerMachine.html#module-note"},{"type":"module","title":"ExMachine.State","doc":"Represents a state in a hierarchical statechart.\n\nA state is the fundamental building block of a statechart and can itself\nbe a statechart (containing hierarchical substates). States can be simple\n(leaf nodes) or composite (containing other states), and they define the\nstructure and behavior of your state machine.","ref":"ExMachine.State.html"},{"type":"module","title":"State Types - ExMachine.State","doc":"ExMachine supports several types of states:\n\n* **Simple states** - Leaf nodes with no substates\n* **Composite states** - Container states with substates\n* **Final states** - Terminal states indicating completion\n* **History states** - Pseudo-states that remember previous configurations","ref":"ExMachine.State.html#module-state-types"},{"type":"module","title":"Fields - ExMachine.State","doc":"* `:initial` - Name of the initial substate (required for composite states)\n* `:substates` - Map of substate name to state definition\n* `:transitions` - Map of event names to transition definitions\n* `:entry` - Optional function executed when entering the state\n* `:exit` - Optional function executed when exiting the state","ref":"ExMachine.State.html#module-fields"},{"type":"module","title":"State Composition - ExMachine.State","doc":"States can be composed in a hierarchical manner, where a composite state\ncontains multiple substates. This allows for modeling complex behaviors\nwith clear organization and state inheritance.","ref":"ExMachine.State.html#module-state-composition"},{"type":"module","title":"Examples - ExMachine.State","doc":"","ref":"ExMachine.State.html#module-examples"},{"type":"module","title":"Simple State - ExMachine.State","doc":"%ExMachine.State{\n      transitions: %{\n        \"start\" => \"working\"\n      },\n      entry: fn context ->\n        IO.puts(\"Entering idle state\")\n        context\n      end\n    }","ref":"ExMachine.State.html#module-simple-state"},{"type":"module","title":"Composite State with Substates - ExMachine.State","doc":"%ExMachine.State{\n      initial: \"connecting\",\n      substates: %{\n        \"connecting\" => %ExMachine.State{\n          transitions: %{\n            \"connected\" => \"authenticated\",\n            \"timeout\" => \"failed\"\n          },\n          entry: fn context ->\n            start_connection_timer(context)\n          end\n        },\n        \"authenticated\" => %ExMachine.State{\n          transitions: %{\n            \"disconnect\" => \"disconnected\"\n          }\n        },\n        \"failed\" => %ExMachine.Final{\n          entry: fn context ->\n            log_connection_failure(context)\n          end\n        }\n      },\n      transitions: %{\n        \"reset\" => \"connecting\"\n      }\n    }","ref":"ExMachine.State.html#module-composite-state-with-substates"},{"type":"module","title":"State with Entry and Exit Actions - ExMachine.State","doc":"%ExMachine.State{\n      entry: fn context ->\n        # Initialize resources, start timers, etc.\n        context\n        |> put(:start_time, System.system_time(:second))\n        |> put(:status, :active)\n      end,\n      exit: fn context ->\n        # Cleanup resources, stop timers, etc.\n        cleanup_resources(context)\n        delete(context, :start_time)\n      end,\n      transitions: %{\n        \"complete\" => \"finished\",\n        \"cancel\" => \"cancelled\"\n      }\n    }","ref":"ExMachine.State.html#module-state-with-entry-and-exit-actions"},{"type":"module","title":"State Definition Best Practices - ExMachine.State","doc":"","ref":"ExMachine.State.html#module-state-definition-best-practices"},{"type":"module","title":"1. Clear State Names - ExMachine.State","doc":"Use descriptive names that clearly indicate the state's purpose:\n\n    # Good\n    \"user_authenticated\"\n    \"payment_processing\"\n    \"document_uploading\"\n\n    # Avoid\n    \"state1\"\n    \"temp\"\n    \"misc\"","ref":"ExMachine.State.html#module-1-clear-state-names"},{"type":"module","title":"2. Proper State Hierarchy - ExMachine.State","doc":"Organize related states under common parent states:\n\n    %ExMachine.State{\n      initial: \"idle\",\n      substates: %{\n        \"idle\" => %{...},\n        \"active\" => %ExMachine.State{\n          initial: \"processing\",\n          substates: %{\n            \"processing\" => %{...},\n            \"waiting\" => %{...},\n            \"complete\" => %{...}\n          }\n        }\n      }\n    }","ref":"ExMachine.State.html#module-2-proper-state-hierarchy"},{"type":"module","title":"3. Meaningful Actions - ExMachine.State","doc":"Keep entry and exit actions focused and idempotent:\n\n    entry: fn context ->\n      # Good: specific, focused action\n      context\n      |> start_session_timer()\n      |> log_state_entry(\"authenticated\")\n    end","ref":"ExMachine.State.html#module-3-meaningful-actions"},{"type":"module","title":"4. Consistent Transition Events - ExMachine.State","doc":"Use consistent event naming across your statechart:\n\n    # Consistent event names\n    \"user_login\", \"user_logout\"\n    \"payment_start\", \"payment_complete\", \"payment_failed\"\n    \"doc_upload\", \"doc_process\", \"doc_complete\"","ref":"ExMachine.State.html#module-4-consistent-transition-events"},{"type":"module","title":"Usage in Statechart Definitions - ExMachine.State","doc":"defmodule WorkflowStatechart do\n      use ExMachine.Statechart,\n        definition: %{\n          initial: \"draft\",\n          states: %{\n            \"draft\" => %ExMachine.State{\n              entry: fn context ->\n                put(context, :created_at, DateTime.utc_now())\n              end,\n              transitions: %{\n                \"submit\" => %{\n                  target: \"review\",\n                  guard: fn context ->\n                    validate_document(context)\n                  end\n                }\n              }\n            },\n            \"review\" => %ExMachine.State{\n              initial: \"pending\",\n              substates: %{\n                \"pending\" => %ExMachine.State{\n                  transitions: %{\n                    \"approve\" => \"approved\",\n                    \"reject\" => \"rejected\"\n                  }\n                },\n                \"approved\" => %ExMachine.Final{},\n                \"rejected\" => %ExMachine.State{\n                  transitions: %{\n                    \"revise\" => \"draft\"\n                  }\n                }\n              }\n            }\n          }\n        }\n    end","ref":"ExMachine.State.html#module-usage-in-statechart-definitions"},{"type":"module","title":"State Lifecycle - ExMachine.State","doc":"When a state machine transitions between states, the following lifecycle\nevents occur in order:\n\n1. **Exit actions** of exited states (most nested first)\n2. **Transition action** (if any)\n3. **Entry actions** of entered states (least nested first)\n\nThis ensures proper cleanup and initialization of state-specific resources.","ref":"ExMachine.State.html#module-state-lifecycle"},{"type":"type","title":"ExMachine.State.t/0","doc":"","ref":"ExMachine.State.html#t:t/0"},{"type":"module","title":"ExMachine.Statechart","doc":"Module for building and validating statechart definitions.\n\nA Statechart is a compiled representation of a hierarchical state machine definition.\nIt provides the foundation for creating and running state machines with complex\nhierarchical structures, transitions, actions, and guards.","ref":"ExMachine.Statechart.html"},{"type":"module","title":"Statechart Structure - ExMachine.Statechart","doc":"A compiled statechart contains:\n\n- `states` - A flattened map of all states in the hierarchy with their full paths\n\nThe statechart compiler takes a nested `State` definition and:\n\n1. Validates the state machine structure\n2. Flattens the hierarchy into addressable state paths\n3. Validates all transitions reference valid states\n4. Ensures there are no duplicate state names in the same scope\n5. Verifies initial states are valid","ref":"ExMachine.Statechart.html#module-statechart-structure"},{"type":"module","title":"State Addressing - ExMachine.Statechart","doc":"States in a statechart are addressed using dot notation paths:\n\n- `\"root\"` - The root state\n- `\"root.playing\"` - A top-level state called \"playing\"\n- `\"root.playing.normal_speed\"` - A nested state \"normal_speed\" inside \"playing\"","ref":"ExMachine.Statechart.html#module-state-addressing"},{"type":"module","title":"Configuration - ExMachine.Statechart","doc":"The active configuration represents which states are currently active.\nDue to hierarchical composition, multiple states can be active simultaneously.\n\nFor example, if a media player is in \"normal speed\" mode:\n- Configuration: `[[\"normal_speed\", \"playing\", \"root\"]]`\n- Active states: \"normal_speed\", \"playing\", and \"root\"","ref":"ExMachine.Statechart.html#module-configuration"},{"type":"module","title":"Building a Statechart - ExMachine.Statechart","doc":"# Define your state machine structure\n    definition = %State{\n      initial: \"idle\",\n      substates: %{\n        \"idle\" => %State{transitions: %{\"start\" => \"running\"}},\n        \"running\" => %State{transitions: %{\"stop\" => \"idle\"}}\n      }\n    }\n\n    # Compile into a statechart\n    statechart = Statechart.build(definition)","ref":"ExMachine.Statechart.html#module-building-a-statechart"},{"type":"module","title":"Validation - ExMachine.Statechart","doc":"The build process performs comprehensive validation:\n\n- All referenced states must exist\n- Initial states must be valid substates\n- No duplicate state names in the same scope\n- Transitions must reference valid target states\n- State hierarchy must be well-formed\n\nIf validation fails, specific exceptions are raised describing the problem.","ref":"ExMachine.Statechart.html#module-validation"},{"type":"module","title":"Usage with using macro - ExMachine.Statechart","doc":"You can also define statecharts using the `__using__` macro:\n\n    defmodule MyMachine do\n      use ExMachine.Statechart\n\n      def definition do\n        %State{\n          # ... your state definition\n        }\n      end\n    end","ref":"ExMachine.Statechart.html#module-usage-with-using-macro"},{"type":"function","title":"ExMachine.Statechart.build/1","doc":"Build and return a `Statechart` struct that contains the compiled and\nvalidated version of the statechart in `definition` argument,\nready to be executed in a Machine.\n\nDuring compilation, Statechart verifies that the definition is valid and raise\nan exception if there is a problem.","ref":"ExMachine.Statechart.html#build/1"},{"type":"function","title":"Examples - ExMachine.Statechart.build/1","doc":"iex> eng = Statechart.build(%State{initial: \"s1\", substates: %{ \"s1\" => %State{}}})\n    iex> Enum.count(eng.states)\n    2\n\n    iex> Statechart.build(\"invalid\")\n    ** (ExMachine.Statechart.InvalidDefinition) Definition \"invalid\" is not valid\n\n    iex> defs = %State{initial: \"invalid_state\", substates: %{ \"s1\" => %State{}}}\n    iex> Statechart.build(defs)\n    ** (ExMachine.Statechart.NotValidInitial) Initial state \"invalid_state\" is not valid or not a descendant of composite state \"root\"","ref":"ExMachine.Statechart.html#build/1-examples"},{"type":"function","title":"ExMachine.Statechart.find_lcca/2","doc":"Return the Least Common Compound Ancestor of `states` list.\n\nLCCA of a list `states` is the lowest (i.e. deepest) state in the state\nhierarchy that has all state in `states` as descendants.\n\nIn other words LCCA is the state `s` such that `s` is a ancestor of all\nstates on `states` list and no descendants of `s` has this property.\n\nReturn `nil` if in the `states` list is present the root state because can't\nexist a state that is parent of root state.\n\nNote that since we are speaking of ancestor (parent or parent\nof a parent, etc.) the LCCA is never a member of `state` list.","ref":"ExMachine.Statechart.html#find_lcca/2"},{"type":"function","title":"ExMachine.Statechart.get_ancestors/2","doc":"Returns the ancestors of `state` (parent of state, parent of parent, etc),\nin form of a list of string, ordered from nearest parent to (and always)\nthe \"root\" state.","ref":"ExMachine.Statechart.html#get_ancestors/2"},{"type":"function","title":"Examples - ExMachine.Statechart.get_ancestors/2","doc":"iex> defs = %State{initial: \"s1\", substates: %{ \"s1\" => %State{ initial: \"s11\", substates: %{ \"s11\" => %State{}}}}}\n    iex> eng = Statechart.build(defs)\n    iex> Statechart.get_ancestors(eng, \"s11\")\n    [\"s1\", \"root\"]","ref":"ExMachine.Statechart.html#get_ancestors/2-examples"},{"type":"function","title":"ExMachine.Statechart.get_ancestors_until/3","doc":"Returns the ancestors of `state` (parent of state, parent of parent, etc),\nin form of a list of string, ordered from nearest parent to (and excluded)\nthe `until` state.","ref":"ExMachine.Statechart.html#get_ancestors_until/3"},{"type":"function","title":"Examples - ExMachine.Statechart.get_ancestors_until/3","doc":"iex> defs = %State{initial: \"s1\", substates: %{ \"s1\" => %State{ initial: \"s11\", substates: %{ \"s11\" => %State{}}}}}\n    iex> eng = Statechart.build(defs)\n    iex> Statechart.get_ancestors_until(eng, \"s11\", \"root\")\n    [\"s1\"]","ref":"ExMachine.Statechart.html#get_ancestors_until/3-examples"},{"type":"function","title":"ExMachine.Statechart.get_descendants/2","doc":"Returns the descendants of `state`, in form of an unordered MapSet of string,\ncontaining all the descendants (children, children of children, etc) of `state`.","ref":"ExMachine.Statechart.html#get_descendants/2"},{"type":"function","title":"Examples - ExMachine.Statechart.get_descendants/2","doc":"iex> defs = %State{initial: \"s1\", substates: %{ \"s1\" => %State{ initial: \"s11\", substates: %{ \"s11\" => %State{}}}}}\n    iex> eng = Statechart.build(defs)\n    iex> Statechart.get_descendants(eng, \"root\")\n    MapSet.new([\"s1\", \"s11\"])","ref":"ExMachine.Statechart.html#get_descendants/2-examples"},{"type":"function","title":"ExMachine.Statechart.get_entering_states/3","doc":"Returns a list of states that must be entered when the machine is entering\nin the state `target`, considering the `lcca`","ref":"ExMachine.Statechart.html#get_entering_states/3"},{"type":"function","title":"ExMachine.Statechart.get_entry_actions/2","doc":"Return the list of enter actions for each state in list `states_list`,\nif defined for the state,  in the same exact order of `states_list`","ref":"ExMachine.Statechart.html#get_entry_actions/2"},{"type":"function","title":"ExMachine.Statechart.get_exit_actions/2","doc":"Return the list of exit actions for each state in list `states_list`,\nif defined for the state, in the same exact order of `states_list`","ref":"ExMachine.Statechart.html#get_exit_actions/2"},{"type":"function","title":"ExMachine.Statechart.get_exiting_states/3","doc":"Returns a list of states that must be exited when the machine is exiting\nfrom the state `source`, considering the `lcca`","ref":"ExMachine.Statechart.html#get_exiting_states/3"},{"type":"function","title":"ExMachine.Statechart.get_initials/2","doc":"Return list of initial states from argument `state` deep to a leaf state.\n\nThe function uses `:initial` key in the `%State{}` definition,\nunless it encounter a history state (to be implemented)","ref":"ExMachine.Statechart.html#get_initials/2"},{"type":"function","title":"Examples - ExMachine.Statechart.get_initials/2","doc":"iex> defs = %State{initial: \"s1\", substates: %{ \"s1\" => %State{ initial: \"s11\", substates: %{ \"s11\" => %State{}}}}}\n    iex> eng = Statechart.build(defs)\n    iex> Statechart.get_initials(eng, \"root\")\n    [\"root\", \"s1\", \"s11\"]\n    iex> Statechart.get_initials(eng, \"s1\")\n    [\"s1\", \"s11\"]","ref":"ExMachine.Statechart.html#get_initials/2-examples"},{"type":"function","title":"ExMachine.Statechart.get_transition_for/3","doc":"Return a transition map if `state` have a transition defined\nto handle `event`, otherwise `nil`","ref":"ExMachine.Statechart.html#get_transition_for/3"},{"type":"function","title":"ExMachine.Statechart.new/0","doc":"","ref":"ExMachine.Statechart.html#new/0"},{"type":"type","title":"ExMachine.Statechart.t/0","doc":"","ref":"ExMachine.Statechart.html#t:t/0"},{"type":"module","title":"ExMachine.Transition","doc":"Defines a state transition with optional guard functions and actions.\n\nTransitions specify how a state machine moves from one state to another in response\nto events. They can include conditional logic (guards) and side effects (actions).","ref":"ExMachine.Transition.html"},{"type":"module","title":"Structure - ExMachine.Transition","doc":"A transition contains:\n\n- `target` - The destination state name\n- `guard` - Optional function that must return `true` for the transition to fire\n- `action` - Optional function to execute when the transition is taken","ref":"ExMachine.Transition.html#module-structure"},{"type":"module","title":"Guard Functions - ExMachine.Transition","doc":"Guards are predicates that determine whether a transition should be taken.\nThey receive the current context and must return a boolean:\n\n    guard: fn context -> context.counter > 10 end\n\nIf multiple transitions from the same state have the same trigger event,\nguards are used to determine which transition should fire.","ref":"ExMachine.Transition.html#module-guard-functions"},{"type":"module","title":"Action Functions - ExMachine.Transition","doc":"Actions are executed when a transition is taken and can modify the context:\n\n    action: fn context -> %{context | counter: context.counter + 1} end\n\nActions should be pure functions that return the new context state.","ref":"ExMachine.Transition.html#module-action-functions"},{"type":"module","title":"Usage - ExMachine.Transition","doc":"Transitions can be defined in several ways:\n\n    # Simple transition (just target state)\n    \"event\" => \"target_state\"\n\n    # Transition with action\n    \"event\" => %Transition{\n      target: \"target_state\",\n      action: fn context -> %{context | value: \"new_value\"} end\n    }\n\n    # Transition with guard and action\n    \"event\" => %Transition{\n      target: \"target_state\",\n      guard: fn context -> context.ready? end,\n      action: fn context -> %{context | attempts: context.attempts + 1} end\n    }","ref":"ExMachine.Transition.html#module-usage"},{"type":"module","title":"Examples - ExMachine.Transition","doc":"# Conditional transition based on context\n    %Transition{\n      target: \"success\",\n      guard: fn context -> context.password == \"secret\" end,\n      action: fn context -> %{context | authenticated: true} end\n    }\n\n    # Transition that modifies context\n    %Transition{\n      target: \"counting\",\n      action: fn context -> Map.update(context, :count, 1, &(&1 + 1)) end\n    }","ref":"ExMachine.Transition.html#module-examples"},{"type":"function","title":"ExMachine.Transition.new/1","doc":"Create a new transition with just a target state.","ref":"ExMachine.Transition.html#new/1"},{"type":"function","title":"Examples - ExMachine.Transition.new/1","doc":"Transition.new(\"target_state\")","ref":"ExMachine.Transition.html#new/1-examples"},{"type":"type","title":"ExMachine.Transition.t/0","doc":"","ref":"ExMachine.Transition.html#t:t/0"},{"type":"exception","title":"ExMachine.Machine.NotRunning","doc":"Raised when dispatching an event to a not running machine","ref":"ExMachine.Machine.NotRunning.html"},{"type":"exception","title":"ExMachine.Statechart.DuplicatedState","doc":"Raised when a state name is used more than once","ref":"ExMachine.Statechart.DuplicatedState.html"},{"type":"exception","title":"ExMachine.Statechart.InvalidDefinition","doc":"Raised when submitted an invalid definition","ref":"ExMachine.Statechart.InvalidDefinition.html"},{"type":"exception","title":"ExMachine.Statechart.NotDefinedState","doc":"Raised when a state name is used that is not defined","ref":"ExMachine.Statechart.NotDefinedState.html"},{"type":"exception","title":"ExMachine.Statechart.NotValidInitial","doc":"Raised when initial state of a composite state is not defined or is not\na descendants of the state","ref":"ExMachine.Statechart.NotValidInitial.html"},{"type":"extras","title":"ExMachine","doc":"# ExMachine\n\n[![Hex Version](https://img.shields.io/hexpm/v/ex_machine.svg)](https://hex.pm/packages/ex_machine)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ex_machine/)\n[![CI](https://github.com/carlotorrese/ex_machine/workflows/CI/badge.svg)](https://github.com/carlotorrese/ex_machine/actions)\n[![License](https://img.shields.io/hexpm/l/ex_machine.svg)](https://github.com/carlotorrese/ex_machine/blob/main/LICENSE)\n[![Development Status](https://img.shields.io/badge/status-alpha-orange.svg)](https://github.com/carlotorrese/ex_machine)\n\n> ⚠️ **Early Development Warning**: This project is in active development and the API may change. Use with caution in production environments.\n\nAn Elixir functional implementation of a finite state machine,\nbased on Statechart.\n\nExMachine is a purely functional implementation of a finite\nstate machine which definition is based on the Statechart formalism\nproposed by David Harel in 1987 and subsequently adopted by\nUnified Modelling Language as its standard for state machine definition.\n\nIt can be used as a simple library, using a function to dispatch an event to the machine,\nreceiving back the new machine, that include the new state and the modified context.\nAlternatively, it can be wrapped in a GenServer to run in a separate process\nas a real and fully independent machine, influenced by events sended to it.\n\nState machine are defined as static structures inside Elixir modules and\ncan be validated and loaded at compile time for better efficiency.\nMoreover State definitions are composable, allowing realization of\nindependent components, based on statechart.","ref":"readme.html"},{"type":"extras","title":"Statechart basics - ExMachine","doc":"The primary feature of statecharts is that\nstates can be organized in a hierarchy:  \na statechart is a state machine where each state\nin the state machine may define its own subordinate state machines,\ncalled substates.\nThose states can again define substates.\n\nMain supported statechart feature supported by ExMachine are:\n\n- [x] Entry and exit actions\n- [x] Transition actions\n- [x] Guard functions\n- [x] Extended state (context)\n- [x] Internal events (run to completion)\n- [x] Final pseudostates\n- [ ] History pseudostate\n- [ ] Choice pseudostates ()\n- [ ] Internal/external transitions ()\n\nFuture implementation:\n\n- [ ] Orthogonal regions i.e. parallel state\n\nNot supported:\n\n- [ ] Do actions","ref":"readme.html#statechart-basics"},{"type":"extras","title":"Finite State Machine - ExMachine","doc":"[Statechart](https://en.wikipedia.org/wiki/Statechart),\nor [Hierarchical State Machine](https://en.wikipedia.org/wiki/UML_state_machine),\nis a formalism to define a particular type of state machine called\n[Finite State Machine (FSM)](https://en.wikipedia.org/wiki/Finite-state_machine).\nA FSM is an abstract machine that can be in exactly one of a finite number\nof states at any given time.\nFSM can change from one state to another in response to some external inputs,\nthis changes are called transitions.\nAn FSM is defined by the complete list of its states, its initial state,\nand the conditions for each transition.","ref":"readme.html#finite-state-machine"},{"type":"extras","title":"State machine definition and execution - ExMachine","doc":"- A way to _*define*_ the machine, listing the complete set of possible\n  states and transitions\n- A way to _*execute*_ the machine, starting it from a specific definition and\n  sending changes to it to observing the consequent evolution.\n\nExMachine supplies both this parts","ref":"readme.html#state-machine-definition-and-execution"},{"type":"extras","title":"States - ExMachine","doc":"In statechart, a state can be either a _simple_ state (i.e., a normal state)\nor a _composite_ state, parent of other substates (children).\nWhen a statechart machine is in a simple state, it is also in it's parent state.\nMoreover, in a statechart, two or more composite state can be active at the\nsame time (orthogonal regions), modelling the situation in which the machine\nis made up of different independent parts, each one interacting with each other.\n\nFor statechart machine we can't say that the machine is in a state,\nbut that it is in a particular _configuration_: a set of one or more\nhierarchy of states, each one from a simple state up to the root state,\nfollowing the parent's tree.\nAll of the states in a configuration are actives at the same time.","ref":"readme.html#states"},{"type":"extras","title":"Context - ExMachine","doc":"Statechart is defined not only by the states but also by a so-called\n_extended state_:\na kind of data structure that models the world (the `context`) in which\nthe machine lives and that influences and is influenced by the\nmachine during it's evolution (transitions).\nSo the configuration of a machine at a given time must contain not only\nall active states but also the context of the machine in that time.\nThe context is the main data structure manipulated by the code executed in\nthe actions by a state machine and the output to the world.","ref":"readme.html#context"},{"type":"extras","title":"Transitions - ExMachine","doc":"A transition is a binary relation between two states of any type, that\nspecifies at which conditions the machine can move to a new configuration.\nTransitions defines how the machine react to a stimulus from the outside world\n(an event) and how it changes the machine itself.","ref":"readme.html#transitions"},{"type":"extras","title":"Events - ExMachine","doc":"Events are the way in which changes are submitted to the machine and in\nwhich this evolves from a state (configuration) to another one.\n\nAn event can induce a change in the state machine if in the statechart\ndefinition exist a transition that declares this event as a trigger.\n\nAn event can also carry some information, called event parameters,\nthat can be used by the machine to decide if the transition must be taken or\nif it must modify the context after the transition is performed.","ref":"readme.html#events"},{"type":"extras","title":"Guards - ExMachine","doc":"The transitions of a machine are influenced either by the event and\nits parameter, that by the current context.\nThese influences are made by particular functions, linked to transitions\ndefinitions, called _guard_.\nThese functions are called when transition is fired by an event,\nand perform a check against event parameters and the context, allowing or\nnegating the transition execution.\nGuard functions never modified directly the context,\nthey return only if the transition must be taken.","ref":"readme.html#guards"},{"type":"extras","title":"Actions - ExMachine","doc":"Statechart defines special functions, called _actions_, that can be performed\neach time a transition happens, changing the configuration.\nThis actions are the only way a machine can change its context and they\ncan also send other events to the machine (internal events),\nthus influencing the final configuration that the machine will take.","ref":"readme.html#actions"},{"type":"extras","title":"Installation - ExMachine","doc":"The package can be installed by adding `ex_machine` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_machine, \"~> 0.1.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExMachine","doc":"ExMachine provides two main ways to use state machines:\n\n1. **Functional approach**: Direct function calls\n2. **Process-based approach**: Using GenServer integration","ref":"readme.html#usage"},{"type":"extras","title":"Basic Example: Simple Traffic Light - ExMachine","doc":"```elixir\ndefmodule TrafficLight do\n  use ExMachine.Statechart\n\n  alias ExMachine.{State, Transition}\n\n  def definition do\n    %State{\n      initial: \"red\",\n      substates: %{\n        \"red\" => %State{\n          entry: fn context ->\n            IO.puts(\"🔴 Red light - STOP\")\n            context\n          end,\n          transitions: %{\n            \"timer\" => \"green\"\n          }\n        },\n        \"green\" => %State{\n          entry: fn context ->\n            IO.puts(\"🟢 Green light - GO\")\n            context\n          end,\n          transitions: %{\n            \"timer\" => \"yellow\"\n          }\n        },\n        \"yellow\" => %State{\n          entry: fn context ->\n            IO.puts(\"🟡 Yellow light - CAUTION\")\n            context\n          end,\n          transitions: %{\n            \"timer\" => \"red\"\n          }\n        }\n      }\n    }\n  end\nend\n\n# Usage\nalias ExMachine.{Machine, Statechart}\n\n# Create and initialize the machine\nstatechart = Statechart.build(TrafficLight.definition())\nmachine = Machine.init(statechart, %{})\n\n# Send events to transition between states\nmachine = Machine.dispatch(machine, \"timer\")  # red -> green\nmachine = Machine.dispatch(machine, \"timer\")  # green -> yellow\nmachine = Machine.dispatch(machine, \"timer\")  # yellow -> red\n```","ref":"readme.html#basic-example-simple-traffic-light"},{"type":"extras","title":"Advanced Example: User Authentication Flow - ExMachine","doc":"```elixir\ndefmodule UserAuth do\n  use ExMachine.Statechart\n\n  alias ExMachine.{State, Transition, Final}\n\n  def definition do\n    %State{\n      initial: \"logged_out\",\n      substates: %{\n        \"logged_out\" => %State{\n          transitions: %{\n            \"login_attempt\" => \"checking_credentials\"\n          }\n        },\n        \"checking_credentials\" => %State{\n          transitions: %{\n            \"success\" => %Transition{\n              target: \"logged_in\",\n              action: &log_successful_login/1\n            },\n            \"failure\" => %Transition{\n              target: \"login_failed\",\n              action: &increment_attempts/1\n            }\n          }\n        },\n        \"login_failed\" => %State{\n          transitions: %{\n            \"retry\" => %Transition{\n              target: \"logged_out\",\n              guard: &can_retry?/1\n            },\n            \"retry\" => \"locked\"  # If guard fails\n          }\n        },\n        \"locked\" => %State{\n          entry: &set_lock_time/1\n        },\n        \"logged_in\" => %State{\n          entry: &clear_attempts/1,\n          transitions: %{\n            \"logout\" => \"logged_out\",\n            \"session_timeout\" => \"logged_out\"\n          }\n        }\n      }\n    }\n  end\n\n  # Action functions\n  defp log_successful_login(context) do\n    context\n    |> Map.put(:last_login, DateTime.utc_now())\n    |> Map.put(:attempts, 0)\n  end\n\n  defp increment_attempts(context) do\n    Map.update(context, :attempts, 1, &(&1 + 1))\n  end\n\n  defp clear_attempts(context) do\n    Map.put(context, :attempts, 0)\n  end\n\n  defp set_lock_time(context) do\n    Map.put(context, :locked_at, DateTime.utc_now())\n  end\n\n  # Guard functions\n  defp can_retry?(context) do\n    Map.get(context, :attempts, 0) < 3\n  end\nend\n\n# Usage example\nstatechart = Statechart.build(UserAuth.definition())\nmachine = Machine.init(statechart, %{attempts: 0})\n\n# Simulate authentication flow\nmachine = Machine.dispatch(machine, \"login_attempt\")\nmachine = Machine.dispatch(machine, \"failure\")      # First failure\nmachine = Machine.dispatch(machine, \"retry\")\nmachine = Machine.dispatch(machine, \"login_attempt\")\nmachine = Machine.dispatch(machine, \"success\")      # Success!\n\nIO.inspect(machine.configuration)  # [[\"logged_in\", \"root\"]]\nIO.inspect(machine.context)        # %{attempts: 0, last_login: ~U[...]}\n```","ref":"readme.html#advanced-example-user-authentication-flow"},{"type":"extras","title":"Hierarchical States Example: Media Player - ExMachine","doc":"```elixir\ndefmodule MediaPlayer do\n  use ExMachine.Statechart\n\n  alias ExMachine.{State, Transition, Final}\n\n  def definition do\n    %State{\n      initial: \"stopped\",\n      substates: %{\n        \"stopped\" => %State{\n          transitions: %{\n            \"play\" => \"playing\"\n          }\n        },\n        \"playing\" => %State{\n          initial: \"normal_speed\",\n          entry: &start_playback/1,\n          exit: &pause_playback/1,\n          substates: %{\n            \"normal_speed\" => %State{\n              transitions: %{\n                \"fast_forward\" => \"fast_forwarding\",\n                \"rewind\" => \"rewinding\"\n              }\n            },\n            \"fast_forwarding\" => %State{\n              entry: &set_speed_2x/1,\n              transitions: %{\n                \"normal\" => \"normal_speed\"\n              }\n            },\n            \"rewinding\" => %State{\n              entry: &set_speed_reverse/1,\n              transitions: %{\n                \"normal\" => \"normal_speed\"\n              }\n            }\n          },\n          transitions: %{\n            \"pause\" => \"paused\",\n            \"stop\" => \"stopped\"\n          }\n        },\n        \"paused\" => %State{\n          transitions: %{\n            \"play\" => \"playing\",\n            \"stop\" => \"stopped\"\n          }\n        }\n      }\n    }\n  end\n\n  # Action functions\n  defp start_playback(context), do: Map.put(context, :status, :playing)\n  defp pause_playback(context), do: Map.put(context, :status, :paused)\n  defp set_speed_2x(context), do: Map.put(context, :speed, 2.0)\n  defp set_speed_reverse(context), do: Map.put(context, :speed, -1.0)\nend\n```","ref":"readme.html#hierarchical-states-example-media-player"},{"type":"extras","title":"Using with GenServer - ExMachine","doc":"For long-running state machines, you can wrap them in a GenServer:\n\n```elixir\ndefmodule PlayerServer do\n  use GenServer\n  alias ExMachine.{Machine, Statechart}\n\n  def start_link(opts \\\\ []) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  def play, do: GenServer.cast(__MODULE__, {:event, \"play\"})\n  def pause, do: GenServer.cast(__MODULE__, {:event, \"pause\"})\n  def stop, do: GenServer.cast(__MODULE__, {:event, \"stop\"})\n\n  def get_state, do: GenServer.call(__MODULE__, :get_state)\n\n  @impl true\n  def init(_opts) do\n    statechart = Statechart.build(MediaPlayer.definition())\n    machine = Machine.init(statechart, %{})\n    {:ok, machine}\n  end\n\n  @impl true\n  def handle_cast({:event, event}, machine) do\n    new_machine = Machine.dispatch(machine, event)\n    {:noreply, new_machine}\n  end\n\n  @impl true\n  def handle_call(:get_state, _from, machine) do\n    {:reply, machine.configuration, machine}\n  end\nend\n\n# Usage\n{:ok, _pid} = PlayerServer.start_link()\nPlayerServer.play()\nPlayerServer.get_state()  # [[\"normal_speed\", \"playing\", \"root\"]]\n```","ref":"readme.html#using-with-genserver"},{"type":"extras","title":"Key Concepts - ExMachine","doc":"- **States**: Define the possible states your machine can be in\n- **Transitions**: Define how to move between states based on events\n- **Actions**: Functions that modify the context during transitions\n- **Guards**: Functions that conditionally allow or prevent transitions\n- **Context**: The extended state that travels with your machine\n- **Events**: Messages that trigger state transitions\n\nFor more examples, check the `lib/samples/` directory in the repository.","ref":"readme.html#key-concepts"},{"type":"extras","title":"Curiosity - ExMachine","doc":"The first machine was officially turned on at 21:26:41 on 2018-10-11, italian time.\nHere is the `iex` session:\n\n```elixir\nErlang/OTP 20 [erts-9.2] [source] [64-bit] [smp:2:2] [ds:2:2:10] [async-threads:10] [hipe] [kernel-poll:false] [dtrace]\n\nInteractive Elixir (1.7.2) - press Ctrl+C to exit (type h() ENTER for help)\niex(1)> m = Machine.init(S0.statechart, %{foo: 0})\n%ExMachine.Machine{\n  configuration: [[\"s11\", \"s1\", \"root\"]],\n  context: %{bar: :baz, foo: 1},\n  statechart: %ExMachine.Statechart{...},\n  macrosteps: [\n    %ExMachine.Macrostep{\n      actions: [&S1.set_foo_one/1, &S1.add_bar_baz/1],\n      entered: [\"root\", \"s1\", \"s11\"],\n      event: nil,\n      exited: [],\n      microsteps: %ExMachine.Microstep{\n        actions: [&S1.set_foo_one/1, &S1.add_bar_baz/1],\n        entered: [\"root\", \"s1\", \"s11\"],\n        event: nil,\n        exited: [],\n        transition: nil\n      },\n      timestamp: ~N[2018-10-11 21:26:41.726698],\n      transitions: []\n    }\n  ],\n  queue: [],\n  running?: true\n}\niex(2)>\n\n```","ref":"readme.html#curiosity"},{"type":"extras","title":"Development Status & Roadmap - ExMachine","doc":"ExMachine is currently in **alpha stage**. While the core functionality is working and tested, the API may undergo changes as we gather feedback from the community.","ref":"readme.html#development-status-roadmap"},{"type":"extras","title":"Current State - ExMachine","doc":"- ✅ Core state machine functionality implemented\n- ✅ Statechart formalism support (hierarchical states, transitions, guards)\n- ✅ GenServer integration for process-based state machines\n- ✅ Comprehensive test suite (30+ tests)\n- ✅ Documentation and examples","ref":"readme.html#current-state"},{"type":"extras","title":"Roadmap - ExMachine","doc":"- [ ] API stabilization based on community feedback\n- [ ] Performance optimizations\n- [ ] Additional guard and action features\n- [ ] More comprehensive examples and tutorials\n- [ ] Integration with other Elixir/OTP patterns","ref":"readme.html#roadmap"},{"type":"extras","title":"Contributing - ExMachine","doc":"We welcome contributions! Please see our [contribution guidelines](CONTRIBUTING.md) and feel free to open issues or pull requests.","ref":"readme.html#contributing"},{"type":"extras","title":"Inspiration - ExMachine","doc":"- [Statecharts: a visual formalism for complex systems (David Harel)](https://ac.els-cdn.com/0167642387900359/1-s2.0-0167642387900359-main.pdf?_tid=a9d41960-080a-49d4-b051-5ed409afb933&acdnat=1538686934_8d122d2d37b601ed2f3c06a462a03fa5)\n- [SCXML](https://www.w3.org/TR/scxml/)\n- [StateX: a state management library for modern web applications](https://github.com/rintoj/statex)\n- [The World of Statechart](https://statecharts.github.io/)\n- [SISMIC Interactive Statechart Model Interpreter and Checker](https://github.com/AlexandreDecan/sismic)\n- [UML State Machine](https://www.omg.org/spec/UML/About-UML/)","ref":"readme.html#inspiration"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.1.3] - 2025-06-09","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Comprehensive internal documentation for all ExMachine modules\n- Detailed module documentation with practical examples for:\n  - `ExMachine.State` - Hierarchical state definitions with best practices\n  - `ExMachine.Final` - Final state usage and examples\n  - `ExMachine.History` - History state types (shallow/deep) with media player example\n  - `ExMachine.Context` - Context management utilities with complete API documentation\n  - `ExMachine.Macrostep` - Macrostep execution tracing with debugging examples\n  - `ExMachine.Microstep` - Atomic step documentation with state lifecycle details\n  - `ExMachine.ServerMachine` - GenServer-based statechart execution (placeholder implementation)","ref":"changelog.html#added"},{"type":"extras","title":"Improved - Changelog","doc":"- Enhanced code examples throughout the documentation\n- Better type specifications and function documentation\n- Improved module organization and cross-references\n- Added debugging and monitoring guidance\n\n## [0.1.2] - 2025-06-09","ref":"changelog.html#improved"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated README documentation and project configuration\n- Refined project settings and documentation\n\n## [0.1.1] - 2025-06-09","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Development status indicators and badges\n- Early development warning in README\n- Comprehensive contributing guidelines (CONTRIBUTING.md)\n- Project roadmap and current state documentation\n- Alpha status indication in package description","ref":"changelog.html#added-1"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated README with development status badges\n- Enhanced documentation for contributors\n\n## [0.1.0-alpha] - 2025-06-09","ref":"changelog.html#changed-1"},{"type":"extras","title":"Added - Changelog","doc":"- **Alpha Release**: Initial release of ExMachine functional state machine library\n- Complete implementation of Statechart-based finite state machine\n- Support for hierarchical states with parent-child relationships\n- Entry and exit actions for state transitions\n- Transition actions with custom logic\n- Guard functions for conditional transitions\n- Extended state management (context)\n- Internal events with run-to-completion semantics\n- Final pseudostates for termination\n- Comprehensive test suite with 30 tests\n- Sample implementations (S0, S1, S2, Authentication)\n- Complete documentation with doctests\n- Modern Elixir 1.16.1 and Erlang 26.2.2 support","ref":"changelog.html#added-2"},{"type":"extras","title":"Technical Details - Changelog","doc":"- Purely functional implementation\n- Compile-time validation of state machine definitions\n- Composable state definitions across modules\n- GenServer integration for process-based execution\n- Macrostep and microstep execution tracking\n- Full compatibility with modern Elixir ecosystem","ref":"changelog.html#technical-details"},{"type":"extras","title":"Dependencies - Changelog","doc":"- ex_doc ~> 0.31 (documentation generation)\n- dialyxir ~> 1.4 (static analysis)\n\n[Unreleased]: https://github.com/YOUR_USERNAME/ex_machine/compare/v0.1.0...HEAD\n[0.1.0]: https://github.com/YOUR_USERNAME/ex_machine/releases/tag/v0.1.0","ref":"changelog.html#dependencies"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2018-2025 Restore srl\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}